@@ -9,9 +9,15 @@ try:
 except:
   # Python 2
   import urlparse
-  from urllib import quote as urlquote
+  def urlquote(s, safe=b'/'):
+    import urllib
+    if isinstance(s, unicode):
+      s = s.encode('utf-8')
+    return urllib.quote(s, safe)
 
 import os.path
+import warnings
+
 
 class AtWikiURI(object):
   def __init__(self, base):
@@ -60,13 +66,21 @@ class AtWikiURI(object):
     else:
       return '{0}/search'.format(self._base)
 
-  def tag(self, tag_name='', index=0):
+  def tags(self, sort='', index=1):
+    return '{}/tag/?sort={}&p={}'.format(self._base, sort, index)
+
+  def tag(self, tag_name='', index=1):
+    if tag_name == '':
+      warnings.warn(
+        'Specifying empty string to tag_name to get URL for list of tags is deprecated.'
+        ' Use .tags() instead.',
+        DeprecationWarning)
     return '{0}/tag/{1}?p={2}'.format(self._base, urlquote(tag_name), index)
 
   def new(self):
     return '{0}/new'.format(self._base)
 
-  def list(self, sort='update', index=0):
+  def list(self, sort='update', index=1):
     """
     ``sort`` can be any of ``update``, ``create``, ``create_desc`` or ``pagename``.
     """
@@ -115,8 +129,23 @@ class AtWikiURI(object):
   def word(self, page_id, keyword):
     return '{0}/?cmd=word&pageid={1}&word={2}&type=normal'.format(self._base, page_id, urlquote(keyword))
 
-  def edit(self, page_id, menu=True):
-    mode = 'editx' if menu else 'editxx'
+  def edit(self, page_id, menu=None, mode=None):
+    if menu is not None:
+      warnings.warn(
+        '`menu` option will be removed in the future release. Use `mode` option instead.',
+        DeprecationWarning)
+    else:
+      menu = True
+
+    if mode is None:
+      mode = 'editx' if menu else 'editxx'
+
+    if mode not in [
+          'pedit',    # standard
+          'editx',    # simple (with menu)
+          'editxx',   # simple (without menu)
+        ]:
+      raise ValueError('invalid mode: {}'.format(mode))
     return '{0}/{1}/{2}.html'.format(self._base, mode, page_id)
 
   def rename(self, page_id):
